@@ -1,10 +1,10 @@
-from __future__ import print_function, division
+from typing import Any, Set
 
 from itertools import permutations
 
 from sympy.combinatorics import Permutation
 from sympy.core import AtomicExpr, Basic, Expr, Dummy, Function, sympify, diff, Pow, Mul, Add, symbols, Tuple
-from sympy.core.compatibility import range, reduce
+from sympy.core.compatibility import reduce
 from sympy.core.numbers import Zero
 from sympy.functions import factorial
 from sympy.matrices import Matrix
@@ -384,7 +384,7 @@ class Point(Basic):
 
     """
     def __init__(self, coord_sys, coords):
-        super(Point, self).__init__()
+        super().__init__()
         self._coord_sys = coord_sys
         self._coords = Matrix(coords)
         self._args = self._coord_sys, self._coords
@@ -401,7 +401,6 @@ class Point(Basic):
 
     @property
     def free_symbols(self):
-        raise NotImplementedError
         return self._coords.free_symbols
 
 
@@ -483,7 +482,7 @@ class BaseScalarField(AtomicExpr):
         return simplify(coords[self._index]).doit()
 
     # XXX Workaround for limitations on the content of args
-    free_symbols = set()
+    free_symbols = set()  # type: Set[Any]
 
     def doit(self):
         return self
@@ -535,17 +534,17 @@ class BaseVectorField(AtomicExpr):
 
     >>> v = BaseVectorField(R2_r, 1)
     >>> pprint(v(s_field))
-    /  d              \|
-    |-----(g(x, xi_2))||
-    \dxi_2            /|xi_2=y
+    / d           \|
+    |---(g(x, xi))||
+    \dxi          /|xi=y
     >>> pprint(v(s_field).rcall(point_r).doit())
      d
     ---(g(x0, y0))
     dy0
     >>> pprint(v(s_field).rcall(point_p))
-    /  d                           \|
-    |-----(g(r0*cos(theta0), xi_2))||
-    \dxi_2                         /|xi_2=r0*sin(theta0)
+    / d                        \|
+    |---(g(r0*cos(theta0), xi))||
+    \dxi                       /|xi=r0*sin(theta0)
 
     """
 
@@ -654,10 +653,10 @@ class Commutator(Expr):
                     res += c1*b1(c2)*b2 - c2*b2(c1)*b1
             return res
         else:
-            return super(Commutator, cls).__new__(cls, v1, v2)
+            return super().__new__(cls, v1, v2)
 
     def __init__(self, v1, v2):
-        super(Commutator, self).__init__()
+        super().__init__()
         self._args = (v1, v2)
         self._v1 = v1
         self._v2 = v2
@@ -704,13 +703,13 @@ class Differential(Expr):
     >>> dg
     d(g(x, y))
     >>> pprint(dg(e_x))
-    /  d              \|
-    |-----(g(xi_1, y))||
-    \dxi_1            /|xi_1=x
+    / d           \|
+    |---(g(xi, y))||
+    \dxi          /|xi=x
     >>> pprint(dg(e_y))
-    /  d              \|
-    |-----(g(x, xi_2))||
-    \dxi_2            /|xi_2=y
+    / d           \|
+    |---(g(x, xi))||
+    \dxi          /|xi=y
 
     Applying the exterior derivative operator twice always results in:
 
@@ -728,10 +727,10 @@ class Differential(Expr):
         if isinstance(form_field, Differential):
             return Zero()
         else:
-            return super(Differential, cls).__new__(cls, form_field)
+            return super().__new__(cls, form_field)
 
     def __init__(self, form_field):
-        super(Differential, self).__init__()
+        super().__init__()
         self._form_field = form_field
         self._args = (self._form_field, )
 
@@ -839,12 +838,12 @@ class TensorProduct(Expr):
         if multifields:
             if len(multifields) == 1:
                 return scalar*multifields[0]
-            return scalar*super(TensorProduct, cls).__new__(cls, *multifields)
+            return scalar*super().__new__(cls, *multifields)
         else:
             return scalar
 
     def __init__(self, *args):
-        super(TensorProduct, self).__init__()
+        super().__init__()
         self._args = args
 
     def __call__(self, *fields):
@@ -953,14 +952,14 @@ class LieDerivative(Expr):
                              ' vector fields. The supplied argument was not a '
                              'vector field.')
         if expr_form_ord > 0:
-            return super(LieDerivative, cls).__new__(cls, v_field, expr)
+            return super().__new__(cls, v_field, expr)
         if expr.atoms(BaseVectorField):
             return Commutator(v_field, expr)
         else:
             return v_field.rcall(expr)
 
     def __init__(self, v_field, expr):
-        super(LieDerivative, self).__init__()
+        super().__init__()
         self._v_field = v_field
         self._expr = expr
         self._args = (self._v_field, self._expr)
@@ -994,7 +993,7 @@ class BaseCovarDerivativeOp(Expr):
     e_x
     """
     def __init__(self, coord_sys, index, christoffel):
-        super(BaseCovarDerivativeOp, self).__init__()
+        super().__init__()
         self._coord_sys = coord_sys
         self._index = index
         self._christoffel = christoffel
@@ -1036,6 +1035,8 @@ class BaseCovarDerivativeOp(Expr):
                       for k in range(v._coord_sys.dim)])
             derivs.append(d)
         to_subs = [wrt_vector(d) for d in d_funcs]
+        # XXX: This substitution can fail when there are Dummy symbols and the
+        # cache is disabled: https://github.com/sympy/sympy/issues/17794
         result = d_result.subs(list(zip(to_subs, derivs)))
 
         # Remove the dummies
@@ -1064,8 +1065,8 @@ class CovarDerivativeOp(Expr):
 
     """
     def __init__(self, wrt, christoffel):
-        super(CovarDerivativeOp, self).__init__()
-        if len(set(v._coord_sys for v in wrt.atoms(BaseVectorField))) > 1:
+        super().__init__()
+        if len({v._coord_sys for v in wrt.atoms(BaseVectorField)}) > 1:
             raise NotImplementedError()
         if contravariant_order(wrt) != 1 or covariant_order(wrt):
             raise ValueError('Covariant derivatives are defined only with '
