@@ -1,13 +1,11 @@
-from __future__ import print_function, division
-
-from sympy.core import S, sympify, diff, oo
-from sympy.core.function import Function, ArgumentIndexError
-from sympy.core.relational import Eq
-from sympy.core.logic import fuzzy_not
-from sympy.polys.polyerrors import PolynomialError
-from sympy.functions.elementary.complexes import im, sign, Abs
-from sympy.functions.elementary.piecewise import Piecewise
+from sympy.core import S, sympify, diff
 from sympy.core.decorators import deprecated
+from sympy.core.function import Function, ArgumentIndexError
+from sympy.core.logic import fuzzy_not
+from sympy.core.relational import Eq, Ne
+from sympy.functions.elementary.complexes import im, sign
+from sympy.functions.elementary.piecewise import Piecewise
+from sympy.polys.polyerrors import PolynomialError
 from sympy.utilities import filldedent
 
 
@@ -17,42 +15,45 @@ from sympy.utilities import filldedent
 
 
 class DiracDelta(Function):
-    """
+    r"""
     The DiracDelta function and its derivatives.
 
+    Explanation
+    ===========
+
     DiracDelta is not an ordinary function. It can be rigorously defined either
     as a distribution or as a measure.
 
-    DiracDelta only makes sense in definite integrals, and in particular, integrals
-    of the form ``Integral(f(x)*DiracDelta(x - x0), (x, a, b))``, where it equals
-    ``f(x0)`` if ``a <= x0 <= b`` and ``0`` otherwise. Formally, DiracDelta acts
-    in some ways like a function that is ``0`` everywhere except at ``0``,
-    but in many ways it also does not. It can often be useful to treat DiracDelta
-    in formal ways, building up and manipulating expressions with delta functions
-    (which may eventually be integrated), but care must be taken to not treat it
-    as a real function.
-    SymPy's ``oo`` is similar. It only truly makes sense formally in certain contexts
-    (such as integration limits), but SymPy allows its use everywhere, and it tries to be
-    consistent with operations on it (like ``1/oo``), but it is easy to get into trouble
-    and get wrong results if ``oo`` is treated too much like a number.
-    Similarly, if DiracDelta is treated too much like a function, it is easy to get wrong
-    or nonsensical results.
+    DiracDelta only makes sense in definite integrals, and in particular,
+    integrals of the form ``Integral(f(x)*DiracDelta(x - x0), (x, a, b))``,
+    where it equals ``f(x0)`` if ``a <= x0 <= b`` and ``0`` otherwise. Formally,
+    DiracDelta acts in some ways like a function that is ``0`` everywhere except
+    at ``0``, but in many ways it also does not. It can often be useful to treat
+    DiracDelta in formal ways, building up and manipulating expressions with
+    delta functions (which may eventually be integrated), but care must be taken
+    to not treat it as a real function. SymPy's ``oo`` is similar. It only
+    truly makes sense formally in certain contexts (such as integration limits),
+    but SymPy allows its use everywhere, and it tries to be consistent with
+    operations on it (like ``1/oo``), but it is easy to get into trouble and get
+    wrong results if ``oo`` is treated too much like a number. Similarly, if
+    DiracDelta is treated too much like a function, it is easy to get wrong or
+    nonsensical results.
 
     DiracDelta function has the following properties:
 
-    1) ``diff(Heaviside(x), x) = DiracDelta(x)``
-    2) ``integrate(DiracDelta(x - a)*f(x),(x, -oo, oo)) = f(a)`` and
-       ``integrate(DiracDelta(x - a)*f(x),(x, a - e, a + e)) = f(a)``
-    3) ``DiracDelta(x) = 0`` for all ``x != 0``
-    4) ``DiracDelta(g(x)) = Sum_i(DiracDelta(x - x_i)/abs(g'(x_i)))``
-       Where ``x_i``-s are the roots of ``g``
-    5) ``DiracDelta(-x) = DiracDelta(x)``
+    1) $\frac{d}{d x} \theta(x) = \delta(x)$
+    2) $\int_{-\infty}^\infty \delta(x - a)f(x)\, dx = f(a)$ and $\int_{a-
+       \epsilon}^{a+\epsilon} \delta(x - a)f(x)\, dx = f(a)$
+    3) $\delta(x) = 0$ for all $x \neq 0$
+    4) $\delta(g(x)) = \sum_i \frac{\delta(x - x_i)}{\|g'(x_i)\|}$ where $x_i$
+       are the roots of $g$
+    5) $\delta(-x) = \delta(x)$
 
-    Derivatives of ``k``-th order of DiracDelta have the following property:
+    Derivatives of ``k``-th order of DiracDelta have the following properties:
 
-    6) ``DiracDelta(x, k) = 0``, for all ``x != 0``
-    7) ``DiracDelta(-x, k) = -DiracDelta(x, k)`` for odd ``k``
-    8) ``DiracDelta(-x, k) = DiracDelta(x, k)`` for even ``k``
+    6) $\delta(x, k) = 0$ for all $x \neq 0$
+    7) $\delta(-x, k) = -\delta(x, k)$ for odd $k$
+    8) $\delta(-x, k) = \delta(x, k)$ for even $k$
 
     Examples
     ========
@@ -83,18 +84,18 @@ class DiracDelta(Function):
     >>> DiracDelta((x**2 - 1)*y).expand(diracdelta=True, wrt=x)
     DiracDelta(x - 1)/(2*Abs(y)) + DiracDelta(x + 1)/(2*Abs(y))
 
-
     See Also
     ========
 
     Heaviside
-    simplify, is_simple
+    sympy.simplify.simplify.simplify, is_simple
     sympy.functions.special.tensor_functions.KroneckerDelta
 
     References
     ==========
 
     .. [1] http://mathworld.wolfram.com/DeltaFunction.html
+
     """
 
     is_real = True
@@ -103,12 +104,15 @@ class DiracDelta(Function):
         """
         Returns the first derivative of a DiracDelta Function.
 
-        The difference between ``diff()`` and ``fdiff()`` is:-
-        ``diff()`` is the user-level function and ``fdiff()`` is an object method.
-        ``fdiff()`` is just a convenience method available in the ``Function`` class.
-        It returns the derivative of the function without considering the chain rule.
-        ``diff(function, x)`` calls ``Function._eval_derivative`` which in turn calls
-        ``fdiff()`` internally to compute the derivative of the function.
+        Explanation
+        ===========
+
+        The difference between ``diff()`` and ``fdiff()`` is: ``diff()`` is the
+        user-level function and ``fdiff()`` is an object method. ``fdiff()`` is
+        a convenience method available in the ``Function`` class. It returns
+        the derivative of the function without considering the chain rule.
+        ``diff(function, x)`` calls ``Function._eval_derivative`` which in turn
+        calls ``fdiff()`` internally to compute the derivative of the function.
 
         Examples
         ========
@@ -144,11 +148,14 @@ class DiracDelta(Function):
         Returns a simplified form or a value of DiracDelta depending on the
         argument passed by the DiracDelta object.
 
-        The ``eval()`` method is automatically called when the ``DiracDelta`` class
-        is about to be instantiated and it returns either some simplified instance
-        or the unevaluated instance depending on the argument passed. In other words,
-        ``eval()`` method is not needed to be called explicitly, it is being called
-        and evaluated once the object is called.
+        Explanation
+        ===========
+
+        The ``eval()`` method is automatically called when the ``DiracDelta``
+        class is about to be instantiated and it returns either some simplified
+        instance or the unevaluated instance depending on the argument passed.
+        In other words, ``eval()`` method is not needed to be called explicitly,
+        it is being called and evaluated once the object is called.
 
         Examples
         ========
@@ -202,46 +209,50 @@ class DiracDelta(Function):
                 Complex part: %s  found in %s .''' % (
                 repr(im(arg)), repr(arg))))
         c, nc = arg.args_cnc()
-        if c and c[0] == -1:
+        if c and c[0] is S.NegativeOne:
             # keep this fast and simple instead of using
             # could_extract_minus_sign
-            if k % 2 == 1:
+            if k.is_odd:
                 return -cls(-arg, k)
-            elif k % 2 == 0:
+            elif k.is_even:
                 return cls(-arg, k) if k else cls(-arg)
 
     @deprecated(useinstead="expand(diracdelta=True, wrt=x)", issue=12859, deprecated_since_version="1.1")
-    def simplify(self, x):
+    def simplify(self, x, **kwargs):
         return self.expand(diracdelta=True, wrt=x)
 
     def _eval_expand_diracdelta(self, **hints):
-        """Compute a simplified representation of the function using
-           property number 4. Pass wrt as a hint to expand the expression
-           with respect to a particular variable.
+        """
+        Compute a simplified representation of the function using
+        property number 4. Pass ``wrt`` as a hint to expand the expression
+        with respect to a particular variable.
 
-           wrt is:
+        Explanation
+        ===========
 
-           - a variable with respect to which a DiracDelta expression will
-           get expanded.
+        ``wrt`` is:
 
-           Examples
-           ========
+        - a variable with respect to which a DiracDelta expression will
+        get expanded.
 
-           >>> from sympy import DiracDelta
-           >>> from sympy.abc import x, y
+        Examples
+        ========
 
-           >>> DiracDelta(x*y).expand(diracdelta=True, wrt=x)
-           DiracDelta(x)/Abs(y)
-           >>> DiracDelta(x*y).expand(diracdelta=True, wrt=y)
-           DiracDelta(y)/Abs(x)
+        >>> from sympy import DiracDelta
+        >>> from sympy.abc import x, y
 
-           >>> DiracDelta(x**2 + x - 2).expand(diracdelta=True, wrt=x)
-           DiracDelta(x - 1)/3 + DiracDelta(x + 2)/3
+        >>> DiracDelta(x*y).expand(diracdelta=True, wrt=x)
+        DiracDelta(x)/Abs(y)
+        >>> DiracDelta(x*y).expand(diracdelta=True, wrt=y)
+        DiracDelta(y)/Abs(x)
 
-           See Also
-           ========
+        >>> DiracDelta(x**2 + x - 2).expand(diracdelta=True, wrt=x)
+        DiracDelta(x - 1)/3 + DiracDelta(x + 2)/3
 
-           is_simple, Diracdelta
+        See Also
+        ========
+
+        is_simple, Diracdelta
 
         """
         from sympy.polys.polyroots import roots
@@ -280,36 +291,36 @@ class DiracDelta(Function):
         return self
 
     def is_simple(self, x):
-        """is_simple(self, x)
+        """
+        Tells whether the argument(args[0]) of DiracDelta is a linear
+        expression in *x*.
 
-           Tells whether the argument(args[0]) of DiracDelta is a linear
-           expression in x.
+        Examples
+        ========
 
-           x can be:
+        >>> from sympy import DiracDelta, cos
+        >>> from sympy.abc import x, y
 
-           - a symbol
+        >>> DiracDelta(x*y).is_simple(x)
+        True
+        >>> DiracDelta(x*y).is_simple(y)
+        True
 
-           Examples
-           ========
+        >>> DiracDelta(x**2 + x - 2).is_simple(x)
+        False
 
-           >>> from sympy import DiracDelta, cos
-           >>> from sympy.abc import x, y
+        >>> DiracDelta(cos(x)).is_simple(x)
+        False
 
-           >>> DiracDelta(x*y).is_simple(x)
-           True
-           >>> DiracDelta(x*y).is_simple(y)
-           True
+        Parameters
+        ==========
 
-           >>> DiracDelta(x**2 + x - 2).is_simple(x)
-           False
+        x : can be a symbol
 
-           >>> DiracDelta(cos(x)).is_simple(x)
-           False
+        See Also
+        ========
 
-           See Also
-           ========
-
-           simplify, Diracdelta
+        sympy.simplify.simplify.simplify, DiracDelta
 
         """
         p = self.args[0].as_poly(x)
@@ -317,34 +328,36 @@ class DiracDelta(Function):
             return p.degree() == 1
         return False
 
-    def _eval_rewrite_as_Piecewise(self, *args):
-        """Represents DiracDelta in a Piecewise form
+    def _eval_rewrite_as_Piecewise(self, *args, **kwargs):
+        """
+        Represents DiracDelta in a piecewise form.
 
-           Examples
-           ========
+        Examples
+        ========
 
-           >>> from sympy import DiracDelta, Piecewise, Symbol, SingularityFunction
-           >>> x = Symbol('x')
+        >>> from sympy import DiracDelta, Piecewise, Symbol, SingularityFunction
+        >>> x = Symbol('x')
 
-           >>> DiracDelta(x).rewrite(Piecewise)
-           Piecewise((DiracDelta(0), Eq(x, 0)), (0, True))
+        >>> DiracDelta(x).rewrite(Piecewise)
+        Piecewise((DiracDelta(0), Eq(x, 0)), (0, True))
 
-           >>> DiracDelta(x - 5).rewrite(Piecewise)
-           Piecewise((DiracDelta(0), Eq(x - 5, 0)), (0, True))
+        >>> DiracDelta(x - 5).rewrite(Piecewise)
+        Piecewise((DiracDelta(0), Eq(x - 5, 0)), (0, True))
 
-           >>> DiracDelta(x**2 - 5).rewrite(Piecewise)
+        >>> DiracDelta(x**2 - 5).rewrite(Piecewise)
            Piecewise((DiracDelta(0), Eq(x**2 - 5, 0)), (0, True))
 
-           >>> DiracDelta(x - 5, 4).rewrite(Piecewise)
-           DiracDelta(x - 5, 4)
+        >>> DiracDelta(x - 5, 4).rewrite(Piecewise)
+        DiracDelta(x - 5, 4)
 
         """
         if len(args) == 1:
             return Piecewise((DiracDelta(0), Eq(args[0], 0)), (0, True))
 
-    def _eval_rewrite_as_SingularityFunction(self, *args):
+    def _eval_rewrite_as_SingularityFunction(self, *args, **kwargs):
         """
-        Returns the DiracDelta expression written in the form of Singularity Functions.
+        Returns the DiracDelta expression written in the form of Singularity
+        Functions.
 
         """
         from sympy.solvers import solve
@@ -377,24 +390,33 @@ class DiracDelta(Function):
 
 
 class Heaviside(Function):
-    """Heaviside Piecewise function
+    r"""
+    Heaviside Piecewise function.
 
-    Heaviside function has the following properties [1]_:
+    Explanation
+    ===========
 
-    1) ``diff(Heaviside(x),x) = DiracDelta(x)``
-                        ``( 0, if x < 0``
-    2) ``Heaviside(x) = < ( undefined if x==0 [1]``
-                        ``( 1, if x > 0``
-    3) ``Max(0,x).diff(x) = Heaviside(x)``
+    Heaviside function has the following properties:
 
-    .. [1] Regarding to the value at 0, Mathematica defines ``H(0) = 1``,
-           but Maple uses ``H(0) = undefined``.  Different application areas
-           may have specific conventions.  For example, in control theory, it
-           is common practice to assume ``H(0) == 0`` to match the Laplace
-           transform of a DiracDelta distribution.
+    1) $\frac{d}{d x} \theta(x) = \delta(x)$
+    2) $\theta(x) = \begin{cases} 0 & \text{for}\: x < 0 \\ \text{undefined} &
+       \text{for}\: x = 0 \\1 & \text{for}\: x > 0 \end{cases}$
+    3) $\frac{d}{d x} \max(x, 0) = \theta(x)$
 
-    To specify the value of Heaviside at x=0, a second argument can be given.
-    Omit this 2nd argument or pass ``None`` to recover the default behavior.
+    Heaviside(x) is printed as $\theta(x)$ with the SymPy LaTeX printer.
+
+    Regarding to the value at 0, Mathematica defines $\theta(0)=1$, but Maple
+    uses $\theta(0) = \text{undefined}$. Different application areas may have
+    specific conventions. For example, in control theory, it is common practice
+    to assume $\theta(0) = 0$ to match the Laplace transform of a DiracDelta
+    distribution.
+
+    To specify the value of Heaviside at ``x=0``, a second argument can be
+    given. Omit this 2nd argument or pass ``None`` to recover the default
+    behavior.
+
+    Examples
+    ========
 
     >>> from sympy import Heaviside, S
     >>> from sympy.abc import x
@@ -417,8 +439,8 @@ class Heaviside(Function):
     References
     ==========
 
-    .. [2] http://mathworld.wolfram.com/HeavisideStepFunction.html
-    .. [3] http://dlmf.nist.gov/1.16#iv
+    .. [1] http://mathworld.wolfram.com/HeavisideStepFunction.html
+    .. [2] http://dlmf.nist.gov/1.16#iv
 
     """
 
@@ -451,10 +473,12 @@ class Heaviside(Function):
             raise ArgumentIndexError(self, argindex)
 
     def __new__(cls, arg, H0=None, **options):
+        if isinstance(H0, Heaviside) and len(H0.args) == 1:
+            H0 = None
+
         if H0 is None:
             return super(cls, cls).__new__(cls, arg, **options)
-        else:
-            return super(cls, cls).__new__(cls, arg, H0, **options)
+        return super(cls, cls).__new__(cls, arg, H0, **options)
 
     @classmethod
     def eval(cls, arg, H0=None):
@@ -462,11 +486,14 @@ class Heaviside(Function):
         Returns a simplified form or a value of Heaviside depending on the
         argument passed by the Heaviside object.
 
-        The ``eval()`` method is automatically called when the ``Heaviside`` class
-        is about to be instantiated and it returns either some simplified instance
-        or the unevaluated instance depending on the argument passed. In other words,
-        ``eval()`` method is not needed to be called explicitly, it is being called
-        and evaluated once the object is called.
+        Explanation
+        ===========
+
+        The ``eval()`` method is automatically called when the ``Heaviside``
+        class is about to be instantiated and it returns either some simplified
+        instance or the unevaluated instance depending on the argument passed.
+        In other words, ``eval()`` method is not needed to be called explicitly,
+        it is being called and evaluated once the object is called.
 
         Examples
         ========
@@ -504,9 +531,9 @@ class Heaviside(Function):
         """
         H0 = sympify(H0)
         arg = sympify(arg)
-        if arg.is_negative:
+        if arg.is_extended_negative:
             return S.Zero
-        elif arg.is_positive:
+        elif arg.is_extended_positive:
             return S.One
         elif arg.is_zero:
             return H0
@@ -515,23 +542,24 @@ class Heaviside(Function):
         elif fuzzy_not(im(arg).is_zero):
             raise ValueError("Function defined only for Real Values. Complex part: %s  found in %s ." % (repr(im(arg)), repr(arg)) )
 
-    def _eval_rewrite_as_Piecewise(self, arg, H0=None):
-        """Represents Heaviside in a Piecewise form
+    def _eval_rewrite_as_Piecewise(self, arg, H0=None, **kwargs):
+        """
+        Represents Heaviside in a Piecewise form.
 
-           Examples
-           ========
+        Examples
+        ========
 
-           >>> from sympy import Heaviside, Piecewise, Symbol, pprint
-           >>> x = Symbol('x')
+        >>> from sympy import Heaviside, Piecewise, Symbol, pprint
+        >>> x = Symbol('x')
 
-           >>> Heaviside(x).rewrite(Piecewise)
-           Piecewise((0, x < 0), (Heaviside(0), Eq(x, 0)), (1, x > 0))
+        >>> Heaviside(x).rewrite(Piecewise)
+        Piecewise((0, x < 0), (Heaviside(0), Eq(x, 0)), (1, x > 0))
 
-           >>> Heaviside(x - 5).rewrite(Piecewise)
-           Piecewise((0, x - 5 < 0), (Heaviside(0), Eq(x - 5, 0)), (1, x - 5 > 0))
+        >>> Heaviside(x - 5).rewrite(Piecewise)
+        Piecewise((0, x - 5 < 0), (Heaviside(0), Eq(x - 5, 0)), (1, x - 5 > 0))
 
-           >>> Heaviside(x**2 - 1).rewrite(Piecewise)
-           Piecewise((0, x**2 - 1 < 0), (Heaviside(0), Eq(x**2 - 1, 0)), (1, x**2 - 1 > 0))
+        >>> Heaviside(x**2 - 1).rewrite(Piecewise)
+        Piecewise((0, x**2 - 1 < 0), (Heaviside(0), Eq(x**2 - 1, 0)), (1, x**2 - 1 > 0))
 
         """
         if H0 is None:
@@ -542,28 +570,33 @@ class Heaviside(Function):
             return Piecewise((0, arg < 0), (1, arg >= 0))
         return Piecewise((0, arg < 0), (H0, Eq(arg, 0)), (1, arg > 0))
 
-    def _eval_rewrite_as_sign(self, arg, H0=None):
-        """Represents the Heaviside function in the form of sign function.
+    def _eval_rewrite_as_sign(self, arg, H0=None, **kwargs):
+        """
+        Represents the Heaviside function in the form of sign function.
+
+        Explanation
+        ===========
+
         The value of the second argument of Heaviside must specify Heaviside(0)
-        = 1/2 for rewritting as sign to be strictly equivalent.  For easier
+        = 1/2 for rewritting as sign to be strictly equivalent. For easier
         usage, we also allow this rewriting when Heaviside(0) is undefined.
 
         Examples
         ========
 
-        >>> from sympy import Heaviside, Symbol, sign
+        >>> from sympy import Heaviside, Symbol, sign, S
         >>> x = Symbol('x', real=True)
 
-        >>> Heaviside(x).rewrite(sign)
+        >>> Heaviside(x, H0=S.Half).rewrite(sign)
         sign(x)/2 + 1/2
 
         >>> Heaviside(x, 0).rewrite(sign)
-        Heaviside(x, 0)
+        Piecewise((sign(x)/2 + 1/2, Ne(x, 0)), (0, True))
 
-        >>> Heaviside(x - 2).rewrite(sign)
+        >>> Heaviside(x - 2, H0=S.Half).rewrite(sign)
         sign(x - 2)/2 + 1/2
 
-        >>> Heaviside(x**2 - 2*x + 1).rewrite(sign)
+        >>> Heaviside(x**2 - 2*x + 1, H0=S.Half).rewrite(sign)
         sign(x**2 - 2*x + 1)/2 + 1/2
 
         >>> y = Symbol('y')
@@ -580,13 +613,19 @@ class Heaviside(Function):
         sign
 
         """
-        if arg.is_real:
-            if H0 is None or H0 == S.Half:
-                return (sign(arg)+1)/2
+        if arg.is_extended_real:
+            pw1 = Piecewise(
+                ((sign(arg) + 1)/2, Ne(arg, 0)),
+                (Heaviside(0, H0=H0), True))
+            pw2 = Piecewise(
+                ((sign(arg) + 1)/2, Eq(Heaviside(0, H0=H0), S(1)/2)),
+                (pw1, True))
+            return pw2
 
-    def _eval_rewrite_as_SingularityFunction(self, args):
+    def _eval_rewrite_as_SingularityFunction(self, args, **kwargs):
         """
-        Returns the Heaviside expression written in the form of Singularity Functions.
+        Returns the Heaviside expression written in the form of Singularity
+        Functions.
 
         """
         from sympy.solvers import solve
