@@ -14,16 +14,14 @@ Computational Mechanics 56.6 (2015): 967-981
 PDF link : http://dilbert.engr.ucdavis.edu/~suku/quadrature/cls-integration.pdf
 """
 
-from __future__ import print_function, division
-
 from functools import cmp_to_key
 
-from sympy.core import S, diff, Expr, Symbol
-from sympy.simplify.simplify import nsimplify
-from sympy.geometry import Segment2D, Polygon, Point, Point2D
 from sympy.abc import x, y, z
-
+from sympy.core import S, diff, Expr, Symbol
+from sympy.core.sympify import _sympify
+from sympy.geometry import Segment2D, Polygon, Point, Point2D
 from sympy.polys.polytools import LC, gcd_list, degree_list
+from sympy.simplify.simplify import nsimplify
 
 
 def polytope_integrate(poly, expr=None, **kwargs):
@@ -51,7 +49,7 @@ def polytope_integrate(poly, expr=None, **kwargs):
     >>> from sympy.geometry.polygon import Polygon
     >>> from sympy.geometry.point import Point
     >>> from sympy.integrals.intpoly import polytope_integrate
-    >>> polygon = Polygon(Point(0,0), Point(0,1), Point(1,1), Point(1,0))
+    >>> polygon = Polygon(Point(0, 0), Point(0, 1), Point(1, 1), Point(1, 0))
     >>> polys = [1, x, y, x*y, x**2*y, x*y**2]
     >>> expr = x*y
     >>> polytope_integrate(polygon, expr)
@@ -62,9 +60,9 @@ def polytope_integrate(poly, expr=None, **kwargs):
     clockwise = kwargs.get('clockwise', False)
     max_degree = kwargs.get('max_degree', None)
 
-    if clockwise is True:
+    if clockwise:
         if isinstance(poly, Polygon):
-            poly = point_sort(poly)
+            poly = Polygon(*point_sort(poly.vertices), evaluate=False)
         else:
             raise TypeError("clockwise=True works for only 2-Polytope"
                             "V-representation input")
@@ -115,8 +113,9 @@ def polytope_integrate(poly, expr=None, **kwargs):
             return result_dict
 
         for poly in expr:
+            poly = _sympify(poly)
             if poly not in result:
-                if poly is S.Zero:
+                if poly.is_zero:
                     result[S.Zero] = S.Zero
                     continue
                 integral_value = S.Zero
@@ -136,7 +135,7 @@ def polytope_integrate(poly, expr=None, **kwargs):
 
 
 def strip(monom):
-    if monom == S.Zero:
+    if monom.is_zero:
         return 0, 0
     elif monom.is_number:
         return monom, 1
@@ -199,7 +198,7 @@ def main_integrate3d(expr, facets, vertices, hp_params, max_degree=None):
                 #  (term, x_degree, y_degree, z_degree, value over boundary)
                 expr, x_d, y_d, z_d, z_index, y_index, x_index, _ = monom
                 degree = x_d + y_d + z_d
-                if b is S.Zero:
+                if b.is_zero:
                     value_over_face = S.Zero
                 else:
                     value_over_face = \
@@ -220,7 +219,7 @@ def main_integrate3d(expr, facets, vertices, hp_params, max_degree=None):
             facet_count = 0
             for i, facet in enumerate(facets):
                 hp = hp_params[i]
-                if hp[1] == S.Zero:
+                if hp[1].is_zero:
                     continue
                 pi = polygon_integrate(facet, hp, i, facets, vertices, expr, deg)
                 poly_contribute += pi *\
@@ -275,7 +274,7 @@ def main_integrate(expr, facets, hp_params, max_degree=None):
                 m, x_d, y_d, _ = monom
                 value = result.get(m, None)
                 degree = S.Zero
-                if b is S.Zero:
+                if b.is_zero:
                     value_over_boundary = S.Zero
                 else:
                     degree = x_d + y_d
@@ -340,7 +339,7 @@ def polygon_integrate(facet, hp_param, index, facets, vertices, expr, degree):
     -25
     """
     expr = S(expr)
-    if expr == S.Zero:
+    if expr.is_zero:
         return S.Zero
     result = S.Zero
     x0 = vertices[facet[0]]
@@ -406,7 +405,8 @@ def lineseg_integrate(polygon, index, line_seg, expr, degree):
     >>> lineseg_integrate(polygon, 0, line_seg, 1, 0)
     5
     """
-    if expr == S.Zero:
+    expr = _sympify(expr)
+    if expr.is_zero:
         return S.Zero
     result = S.Zero
     x0 = line_seg[0]
@@ -457,7 +457,8 @@ def integration_reduction(facets, index, a, b, expr, dims, degree):
     >>> integration_reduction(facets, 0, a, b, 1, (x, y), 0)
     5
     """
-    if expr == S.Zero:
+    expr = _sympify(expr)
+    if expr.is_zero:
         return expr
 
     value = S.Zero
@@ -743,7 +744,7 @@ def hyperplane_parameters(poly, vertices=None):
             normal = cross_product(v1, v2, v3)
             b = sum([normal[j] * v1[j] for j in range(0, 3)])
             fac = gcd_list(normal)
-            if fac is S.Zero:
+            if fac.is_zero:
                 fac = 1
             normal = [j / fac for j in normal]
             b = b / fac
@@ -815,14 +816,14 @@ def best_origin(a, b, lineseg, expr):
     def x_axis_cut(ls):
         """Returns the point where the input line segment
         intersects the x-axis.
-        Parameters:
-        ===========
+        Parameters
+        ==========
         ls : Line segment
         """
         p, q = ls.points
-        if p.y == S.Zero:
+        if p.y.is_zero:
             return tuple(p)
-        elif q.y == S.Zero:
+        elif q.y.is_zero:
             return tuple(q)
         elif p.y/q.y < S.Zero:
             return p.y * (p.x - q.x)/(q.y - p.y) + p.x, S.Zero
@@ -832,14 +833,14 @@ def best_origin(a, b, lineseg, expr):
     def y_axis_cut(ls):
         """Returns the point where the input line segment
         intersects the y-axis.
-        Parameters:
-        ===========
+        Parameters
+        ==========
         ls : Line segment
         """
         p, q = ls.points
-        if p.x == S.Zero:
+        if p.x.is_zero:
             return tuple(p)
-        elif q.x == S.Zero:
+        elif q.x.is_zero:
             return tuple(q)
         elif p.x/q.x < S.Zero:
             return S.Zero, p.x * (p.y - q.y)/(q.x - p.x) + p.y
@@ -855,12 +856,12 @@ def best_origin(a, b, lineseg, expr):
     if len(gens) > 1:
         # Special case for vertical and horizontal lines
         if len(gens) == 2:
-            if a[0] == S.Zero:
+            if a[0] == 0:
                 if y_axis_cut(lineseg):
                     return S.Zero, b/a[1]
                 else:
                     return a1, b1
-            elif a[1] == S.Zero:
+            elif a[1] == 0:
                 if x_axis_cut(lineseg):
                     return b/a[0], S.Zero
                 else:
@@ -926,8 +927,8 @@ def decompose(expr, separate=False):
     ==========
     expr : Polynomial(SymPy expression)
 
-    Optional Parameters :
-    ---------------------
+    Optional Parameters:
+    --------------------
     separate : If True then simply return a list of the constituent monomials
                If not then break up the polynomial into constituent homogeneous
                polynomials.
@@ -1009,35 +1010,36 @@ def point_sort(poly, normal=None, clockwise=True):
     >>> point_sort([Point(0, 0), Point(1, 0), Point(1, 1)])
     [Point2D(1, 1), Point2D(1, 0), Point2D(0, 0)]
     """
-    n = len(poly)
+    pts = poly.vertices if isinstance(poly, Polygon) else poly
+    n = len(pts)
     if n < 2:
-        return poly
+        return list(pts)
 
-    order = S(1) if clockwise else S(-1)
-    dim = len(poly[0])
+    order = S.One if clockwise else S.NegativeOne
+    dim = len(pts[0])
     if dim == 2:
-        center = Point(sum(map(lambda vertex: vertex.x, poly)) / n,
-                       sum(map(lambda vertex: vertex.y, poly)) / n)
+        center = Point(sum(map(lambda vertex: vertex.x, pts)) / n,
+                        sum(map(lambda vertex: vertex.y, pts)) / n)
     else:
-        center = Point(sum(map(lambda vertex: vertex.x, poly)) / n,
-                       sum(map(lambda vertex: vertex.y, poly)) / n,
-                       sum(map(lambda vertex: vertex.z, poly)) / n)
+        center = Point(sum(map(lambda vertex: vertex.x, pts)) / n,
+                        sum(map(lambda vertex: vertex.y, pts)) / n,
+                        sum(map(lambda vertex: vertex.z, pts)) / n)
 
     def compare(a, b):
         if a.x - center.x >= S.Zero and b.x - center.x < S.Zero:
             return -order
-        elif a.x - center.x < S.Zero and b.x - center.x >= S.Zero:
+        elif a.x - center.x < 0 and b.x - center.x >= 0:
             return order
-        elif a.x - center.x == S.Zero and b.x - center.x == S.Zero:
-            if a.y - center.y >= S.Zero or b.y - center.y >= S.Zero:
+        elif a.x - center.x == 0 and b.x - center.x == 0:
+            if a.y - center.y >= 0 or b.y - center.y >= 0:
                 return -order if a.y > b.y else order
             return -order if b.y > a.y else order
 
         det = (a.x - center.x) * (b.y - center.y) -\
               (b.x - center.x) * (a.y - center.y)
-        if det < S.Zero:
+        if det < 0:
             return -order
-        elif det > S.Zero:
+        elif det > 0:
             return order
 
         first = (a.x - center.x) * (a.x - center.x) +\
@@ -1049,14 +1051,12 @@ def point_sort(poly, normal=None, clockwise=True):
     def compare3d(a, b):
         det = cross_product(center, a, b)
         dot_product = sum([det[i] * normal[i] for i in range(0, 3)])
-        if dot_product < S.Zero:
+        if dot_product < 0:
             return -order
-        elif dot_product > S.Zero:
+        elif dot_product > 0:
             return order
 
-    if dim == 2:
-        return sorted(poly, key=cmp_to_key(compare))
-    return sorted(poly, key=cmp_to_key(compare3d))
+    return sorted(pts, key=cmp_to_key(compare if dim==2 else compare3d))
 
 
 def norm(point):
@@ -1075,7 +1075,7 @@ def norm(point):
     >>> norm(Point(2, 7))
     sqrt(53)
     """
-    half = S(1)/2
+    half = S.Half
     if isinstance(point, (list, tuple)):
         return sum([coord ** 2 for coord in point]) ** half
     elif isinstance(point, Point):
