@@ -1,8 +1,9 @@
 from sympy.external import import_module
+from sympy.utilities.decorator import doctest_depends_on
 
 from .errors import LaTeXParsingError  # noqa
 
-
+@doctest_depends_on(modules=('antlr4',))
 def parse_latex(s):
     r"""Converts the string ``s`` to a SymPy ``Expr``
 
@@ -18,17 +19,17 @@ def parse_latex(s):
     Examples
     ========
 
-    >>> from sympy.parsing.latex import parse_latex  # doctest: +SKIP
-    >>> expr = parse_latex(r"\frac {1 + \sqrt {\a}} {\b}")  # doctest: +SKIP
-    >>> expr  # doctest: +SKIP
+    >>> from sympy.parsing.latex import parse_latex
+    >>> expr = parse_latex(r"\frac {1 + \sqrt {\a}} {\b}")
+    >>> expr
     (sqrt(a) + 1)/b
-    >>> expr.evalf(4, subs=dict(a=5, b=2))  # doctest: +SKIP
+    >>> expr.evalf(4, subs=dict(a=5, b=2))
     1.618
     """
 
     _latex = import_module(
         'sympy.parsing.latex._parse_latex_antlr',
-        __import__kwargs={'fromlist': ['X']})
+        import_kwargs={'fromlist': ['X']})
 
     if _latex is not None:
         return _latex.parse_latex(s)
