@@ -2,12 +2,12 @@
 
 from __future__ import print_function, division
 
-from sympy.polys.polyutils import parallel_dict_from_basic
-from sympy.polys.polyoptions import build_options
-from sympy.polys.domains import ZZ, QQ, RR, EX
-from sympy.polys.domains.realfield import RealField
-from sympy.utilities import public
 from sympy.core import sympify
+from sympy.polys.domains import ZZ, QQ, EX
+from sympy.polys.domains.realfield import RealField
+from sympy.polys.polyoptions import build_options
+from sympy.polys.polyutils import parallel_dict_from_basic
+from sympy.utilities import public
 
 
 def _construct_simple(coeffs, opt):
