@@ -2,7 +2,7 @@
 
 from __future__ import print_function, division
 
-from typing import Any, Optional, Type
+from typing import Any, Optional
 
 from sympy.core import Basic, sympify
 from sympy.core.compatibility import HAS_GMPY, is_sequence
@@ -17,7 +17,7 @@ from sympy.utilities import default_sort_key, public
 class Domain(object):
     """Represents an abstract domain. """
 
-    dtype = None  # type: Optional[Type]
+    dtype = None  ## type: Optional[Type]
     zero = None  # type: Optional[Any]
     one = None  # type: Optional[Any]
 
