@@ -1,4 +1,3 @@
-import warnings
 from sympy.core import (
     S, pi, oo, symbols, Rational, Integer, Float, Mod, GoldenRatio, EulerGamma, Catalan,
     Lambda, Dummy, Eq, nan, Mul, Pow
@@ -11,8 +10,8 @@ from sympy.functions import (
 from sympy.sets import Range
 from sympy.logic import ITE
 from sympy.codegen import For, aug_assign, Assignment
-from sympy.utilities.pytest import raises, XFAIL
-from sympy.printing.ccode import CCodePrinter, C89CodePrinter, C99CodePrinter, get_math_macros
+from sympy.testing.pytest import raises, XFAIL
+from sympy.printing.ccode import C89CodePrinter, C99CodePrinter, get_math_macros
 from sympy.codegen.ast import (
     AddAugmentedAssignment, Element, Type, FloatType, Declaration, Pointer, Variable, value_const, pointer_const,
     While, Scope, Print, FunctionPrototype, FunctionDefinition, FunctionCall, Return,
@@ -21,7 +20,6 @@ from sympy.codegen.ast import (
 from sympy.codegen.cfunctions import expm1, log1p, exp2, log2, fma, log10, Cbrt, hypot, Sqrt
 from sympy.codegen.cnodes import restrict
 from sympy.utilities.lambdify import implemented_function
-from sympy.utilities.exceptions import SymPyDeprecationWarning
 from sympy.tensor import IndexedBase, Idx
 from sympy.matrices import Matrix, MatrixSymbol
 
@@ -73,6 +71,14 @@ def test_ccode_Max():
     assert ccode(Max(x,x*x),user_functions={"Max":"my_max", "Pow":"my_pow"}) == 'my_max(x, my_pow(x, 2))'
 
 
+def test_ccode_Min_performance():
+    #Shouldn't take more than a few seconds
+    big_min = Min(*symbols('a[0:50]'))
+    for curr_standard in ('c89', 'c99', 'c11'):
+        output = ccode(big_min, standard=curr_standard)
+        assert output.count('(') == output.count(')')
+
+
 def test_ccode_constants_mathh():
     assert ccode(exp(1)) == "M_E"
     assert ccode(pi) == "M_PI"
@@ -291,28 +297,24 @@ def test_ccode_Indexed():
     A = IndexedBase('A')[i, j]
     B = IndexedBase('B')[i, j, k]
 
-    with warnings.catch_warnings():
-        warnings.filterwarnings("ignore", category=SymPyDeprecationWarning)
-        p = CCodePrinter()
-        p._not_c = set()
+    p = C99CodePrinter()
 
-        assert p._print_Indexed(x) == 'x[j]'
-        assert p._print_Indexed(A) == 'A[%s]' % (m*i+j)
-        assert p._print_Indexed(B) == 'B[%s]' % (i*o*m+j*o+k)
-        assert p._not_c == set()
+    assert p._print_Indexed(x) == 'x[j]'
+    assert p._print_Indexed(A) == 'A[%s]' % (m*i+j)
+    assert p._print_Indexed(B) == 'B[%s]' % (i*o*m+j*o+k)
 
-        A = IndexedBase('A', shape=(5,3))[i, j]
-        assert p._print_Indexed(A) == 'A[%s]' % (3*i + j)
+    A = IndexedBase('A', shape=(5,3))[i, j]
+    assert p._print_Indexed(A) == 'A[%s]' % (3*i + j)
 
-        A = IndexedBase('A', shape=(5,3), strides='F')[i, j]
-        assert ccode(A) == 'A[%s]' % (i + 5*j)
+    A = IndexedBase('A', shape=(5,3), strides='F')[i, j]
+    assert ccode(A) == 'A[%s]' % (i + 5*j)
 
-        A = IndexedBase('A', shape=(29,29), strides=(1, s), offset=o)[i, j]
-        assert ccode(A) == 'A[o + s*j + i]'
+    A = IndexedBase('A', shape=(29,29), strides=(1, s), offset=o)[i, j]
+    assert ccode(A) == 'A[o + s*j + i]'
 
-        Abase = IndexedBase('A', strides=(s, m, n), offset=o)
-        assert ccode(Abase[i, j, k]) == 'A[m*j + n*k + o + s*i]'
-        assert ccode(Abase[2, 3, k]) == 'A[3*m + n*k + o + 2*s]'
+    Abase = IndexedBase('A', strides=(s, m, n), offset=o)
+    assert ccode(Abase[i, j, k]) == 'A[m*j + n*k + o + s*i]'
+    assert ccode(Abase[2, 3, k]) == 'A[3*m + n*k + o + 2*s]'
 
 
 def test_Element():
@@ -588,16 +590,6 @@ def test_ccode_standard():
     assert ccode(float('nan'), standard='c99') == 'NAN'
 
 
-def test_CCodePrinter():
-    with warnings.catch_warnings():
-        warnings.filterwarnings("error", category=SymPyDeprecationWarning)
-        with raises(SymPyDeprecationWarning):
-            CCodePrinter()
-    with warnings.catch_warnings():
-        warnings.filterwarnings("ignore", category=SymPyDeprecationWarning)
-        assert CCodePrinter().language == 'C'
-
-
 def test_C89CodePrinter():
     c89printer = C89CodePrinter()
     assert c89printer.language == 'C'
@@ -778,14 +770,7 @@ def test_MatrixElement_printing():
     assert(ccode(3 * A[0, 0]) == "3*A[0]")
 
     F = C[0, 0].subs(C, A - B)
-    assert(ccode(F) == "(-B + A)[0]")
-
-
-def test_subclass_CCodePrinter():
-    # issue gh-12687
-    class MySubClass(CCodePrinter):
-        pass
-
+    assert(ccode(F) == "(A - B)[0]")
 
 def test_ccode_math_macros():
     assert ccode(z + exp(1)) == 'z + M_E'
