@@ -3,10 +3,11 @@ Main Random Variables Module
 
 Defines abstract random variable type.
 Contains interfaces for probability space object (PSpace) as well as standard
-operators, P, E, sample, density, where
+operators, P, E, sample, density, where, quantile
 
 See Also
 ========
+
 sympy.stats.crv
 sympy.stats.frv
 sympy.stats.rv_interface
@@ -14,16 +15,18 @@ sympy.stats.rv_interface
 
 from __future__ import print_function, division
 
-from sympy import (Basic, S, Expr, Symbol, Tuple, And, Add, Eq, lambdify,
-        Equality, Lambda, sympify, Dummy, Ne, KroneckerDelta,
-        DiracDelta, Mul, Indexed)
-from sympy.core.relational import Relational
-from sympy.core.compatibility import string_types
-from sympy.logic.boolalg import Boolean
-from sympy.solvers.solveset import solveset
-from sympy.sets.sets import FiniteSet, ProductSet, Intersection
-from sympy.abc import x
+from typing import Tuple as tTuple
 
+from sympy import (Basic, S, Expr, Symbol, Tuple, And, Add, Eq, lambdify,
+                   Equality, Lambda, sympify, Dummy, Ne, KroneckerDelta,
+                   DiracDelta, Mul, Indexed, MatrixSymbol, Function)
+from sympy.core.relational import Relational
+from sympy.core.sympify import _sympify
+from sympy.logic.boolalg import Boolean
+from sympy.sets.sets import FiniteSet, ProductSet, Intersection
+from sympy.solvers.solveset import solveset
+
+x = Symbol('x')
 
 class RandomDomain(Basic):
     """
@@ -31,6 +34,7 @@ class RandomDomain(Basic):
 
     See Also
     ========
+
     sympy.stats.crv.ContinuousDomain
     sympy.stats.frv.FiniteDomain
     """
@@ -65,6 +69,7 @@ class SingleDomain(RandomDomain):
 
     See Also
     ========
+
     sympy.stats.crv.SingleContinuousDomain
     sympy.stats.frv.SingleFiniteDomain
     """
@@ -93,6 +98,7 @@ class ConditionalDomain(RandomDomain):
 
     See Also
     ========
+
     sympy.stats.crv.ConditionalContinuousDomain
     sympy.stats.frv.ConditionalFiniteDomain
     """
@@ -131,14 +137,15 @@ class PSpace(Basic):
 
     See Also
     ========
+
     sympy.stats.crv.ContinuousPSpace
     sympy.stats.frv.FinitePSpace
     """
 
-    is_Finite = None
-    is_Continuous = None
-    is_Discrete = None
-    is_real = None
+    is_Finite = None  # type: bool
+    is_Continuous = None  # type: bool
+    is_Discrete = None  # type: bool
+    is_real = None  # type: bool
 
     @property
     def domain(self):
@@ -178,7 +185,7 @@ class SinglePSpace(PSpace):
     attributed to a single variable/symbol.
     """
     def __new__(cls, s, distribution):
-        if isinstance(s, string_types):
+        if isinstance(s, str):
             s = Symbol(s)
         if not isinstance(s, Symbol):
             raise TypeError("s should have been string or Symbol")
@@ -263,13 +270,35 @@ class RandomSymbol(Expr):
     def is_commutative(self):
         return self.symbol.is_commutative
 
-    def _hashable_content(self):
-        return self.pspace, self.symbol
-
     @property
     def free_symbols(self):
         return {self}
 
+class RandomIndexedSymbol(RandomSymbol):
+
+    def __new__(cls, idx_obj, pspace=None):
+        if not isinstance(idx_obj, (Indexed, Function)):
+            raise TypeError("An Function or Indexed object is expected not %s"%(idx_obj))
+        return Basic.__new__(cls, idx_obj, pspace)
+
+    symbol = property(lambda self: self.args[0])
+    name = property(lambda self: str(self.args[0]))
+
+    @property
+    def key(self):
+        if isinstance(self.symbol, Indexed):
+            return self.symbol.args[1]
+        elif isinstance(self.symbol, Function):
+            return self.symbol.args[0]
+
+class RandomMatrixSymbol(MatrixSymbol):
+    def __new__(cls, symbol, n, m, pspace=None):
+        n, m = _sympify(n), _sympify(m)
+        symbol = _symbol_converter(symbol)
+        return Basic.__new__(cls, symbol, n, m, pspace)
+
+    symbol = property(lambda self: self.args[0])
+    pspace = property(lambda self: self.args[3])
 
 class ProductPSpace(PSpace):
     """
@@ -278,6 +307,7 @@ class ProductPSpace(PSpace):
 
     See Also
     ========
+
     sympy.stats.rv.IndependentProductPSpace
     sympy.stats.joint_rv.JointPSpace
     """
@@ -356,9 +386,9 @@ class IndependentProductPSpace(ProductPSpace):
     def density(self):
         raise NotImplementedError("Density not available for ProductSpaces")
 
-    def sample(self):
-        return dict([(k, v) for space in self.spaces
-            for k, v in space.sample().items()])
+    def sample(self, size=()):
+        return {k: v for space in self.spaces
+            for k, v in space.sample().items()}
 
     def probability(self, condition, **kwargs):
         cond_inv = False
@@ -367,11 +397,10 @@ class IndependentProductPSpace(ProductPSpace):
             cond_inv = True
         expr = condition.lhs - condition.rhs
         rvs = random_symbols(expr)
-        z = Dummy('z', real=True, Finite=True)
         dens = self.compute_density(expr)
         if any([pspace(rv).is_Continuous for rv in rvs]):
-            from sympy.stats.crv import (ContinuousDistributionHandmade,
-                SingleContinuousPSpace)
+            from sympy.stats.crv import SingleContinuousPSpace
+            from sympy.stats.crv_types import ContinuousDistributionHandmade
             if expr in self.values:
                 # Marginalize all other random symbols out of the density
                 randomsymbols = tuple(set(self.values) - frozenset([expr]))
@@ -379,23 +408,26 @@ class IndependentProductPSpace(ProductPSpace):
                 pdf = self.domain.integrate(self.pdf, symbols, **kwargs)
                 return Lambda(expr.symbol, pdf)
             dens = ContinuousDistributionHandmade(dens)
+            z = Dummy('z', real=True)
             space = SingleContinuousPSpace(z, dens)
             result = space.probability(condition.__class__(space.value, 0))
         else:
-            from sympy.stats.drv import (DiscreteDistributionHandmade,
-                SingleDiscretePSpace)
+            from sympy.stats.drv import SingleDiscretePSpace
+            from sympy.stats.drv_types import DiscreteDistributionHandmade
             dens = DiscreteDistributionHandmade(dens)
+            z = Dummy('z', integer=True)
             space = SingleDiscretePSpace(z, dens)
             result = space.probability(condition.__class__(space.value, 0))
         return result if not cond_inv else S.One - result
 
     def compute_density(self, expr, **kwargs):
-        z = Dummy('z', real=True, finite=True)
         rvs = random_symbols(expr)
         if any(pspace(rv).is_Continuous for rv in rvs):
+            z = Dummy('z', real=True)
             expr = self.compute_expectation(DiracDelta(expr - z),
              **kwargs)
         else:
+            z = Dummy('z', integer=True)
             expr = self.compute_expectation(KroneckerDelta(expr, z),
              **kwargs)
         return Lambda(z, expr)
@@ -423,7 +455,9 @@ class IndependentProductPSpace(ProductPSpace):
             replacement  = {rv: Dummy(str(rv)) for rv in self.symbols}
             norm = domain.compute_expectation(self.pdf, **kwargs)
             pdf = self.pdf / norm.xreplace(replacement)
-            density = Lambda(domain.symbols, pdf)
+            # XXX: Converting symbols from set to tuple. The order matters to
+            # Lambda though so we shouldn't be starting with a set here...
+            density = Lambda(tuple(domain.symbols), pdf)
 
         return space(domain, density)
 
@@ -439,8 +473,6 @@ class ProductDomain(RandomDomain):
     is_ProductDomain = True
 
     def __new__(cls, *domains):
-        symbols = sumsets([domain.symbols for domain in domains])
-
         # Flatten any product of products
         domains2 = []
         for domain in domains:
@@ -478,7 +510,7 @@ class ProductDomain(RandomDomain):
 
     @property
     def set(self):
-        return ProductSet(domain.set for domain in self.domains)
+        return ProductSet(*(domain.set for domain in self.domains))
 
     def __contains__(self, other):
         # Split event into each subdomain
@@ -501,9 +533,12 @@ def random_symbols(expr):
     """
     Returns all RandomSymbols within a SymPy Expression.
     """
-    try:
-        return list(expr.atoms(RandomSymbol))
-    except AttributeError:
+    atoms = getattr(expr, 'atoms', None)
+    if atoms is not None:
+        comp = lambda rv: rv.symbol.name
+        l = list(atoms(RandomSymbol))
+        return sorted(l, key=comp)
+    else:
         return []
 
 
@@ -523,8 +558,12 @@ def pspace(expr):
     True
     """
     expr = sympify(expr)
-    if isinstance(expr, RandomSymbol) and expr.pspace != None:
+    if isinstance(expr, RandomSymbol) and expr.pspace is not None:
         return expr.pspace
+    if expr.has(RandomMatrixSymbol):
+        rm = list(expr.atoms(RandomMatrixSymbol))[0]
+        return rm.pspace
+
     rvs = random_symbols(expr)
     if not rvs:
         raise ValueError("Expression containing Random Variable expected, not %s" % (expr))
@@ -617,7 +656,15 @@ def given(expr, condition=None, **kwargs):
             if temp == True:
                 return True
             if temp != False:
-                sums += expr.subs(rv, res)
+                # XXX: This seems nonsensical but preserves existing behaviour
+                # after the change that Relational is no longer a subclass of
+                # Expr. Here expr is sometimes Relational and sometimes Expr
+                # but we are trying to add them with +=. This needs to be
+                # fixed somehow.
+                if sums == 0 and isinstance(expr, Relational):
+                    sums = expr.subs(rv, res)
+                else:
+                    sums += expr.subs(rv, res)
         if sums == 0:
             return False
         return sums
@@ -671,6 +718,9 @@ def expectation(expr, condition=None, numsamples=None, evaluate=True, **kwargs):
     if numsamples:  # Computing by monte carlo sampling?
         return sampling_E(expr, condition, numsamples=numsamples)
 
+    if expr.has(RandomIndexedSymbol):
+        return pspace(expr).compute_expectation(expr, condition, evaluate, **kwargs)
+
     # Create new expr and recompute E
     if condition is not None:  # If there is a condition
         return expectation(given(expr, condition), evaluate=evaluate)
@@ -724,6 +774,20 @@ def probability(condition, given_condition=None, numsamples=None,
     condition = sympify(condition)
     given_condition = sympify(given_condition)
 
+    if condition.has(RandomIndexedSymbol):
+        return pspace(condition).probability(condition, given_condition, evaluate, **kwargs)
+
+    if isinstance(given_condition, RandomSymbol):
+        condrv = random_symbols(condition)
+        if len(condrv) == 1 and condrv[0] == given_condition:
+            from sympy.stats.frv_types import BernoulliDistribution
+            return BernoulliDistribution(probability(condition), 0, 1)
+        if any([dependent(rv, given_condition) for rv in condrv]):
+            from sympy.stats.symbolic_probability import Probability
+            return Probability(condition, given_condition)
+        else:
+            return probability(condition)
+
     if given_condition is not None and \
             not isinstance(given_condition, (Relational, Boolean)):
         raise ValueError("%s is not a relational or combination of relationals"
@@ -765,7 +829,12 @@ class Density(Basic):
 
     def doit(self, evaluate=True, **kwargs):
         from sympy.stats.joint_rv import JointPSpace
+        from sympy.stats.frv import SingleFiniteDistribution
         expr, condition = self.expr, self.condition
+        if _sympify(expr).has(RandomMatrixSymbol):
+            return pspace(expr).compute_density(expr)
+        if isinstance(expr, SingleFiniteDistribution):
+            return expr.dict
         if condition is not None:
             # Recompute on new conditional expr
             expr = given(expr, condition, **kwargs)
@@ -948,28 +1017,42 @@ def where(condition, given_condition=None, **kwargs):
     return pspace(condition).where(condition, **kwargs)
 
 
-def sample(expr, condition=None, **kwargs):
+def sample(expr, condition=None, size=(), **kwargs):
     """
     A realization of the random expression
 
     Examples
     ========
 
-    >>> from sympy.stats import Die, sample
+    >>> from sympy.stats import Die, sample, Normal
     >>> X, Y, Z = Die('X', 6), Die('Y', 6), Die('Z', 6)
 
     >>> die_roll = sample(X + Y + Z) # A random realization of three dice
+    >>> N = Normal('N', 3, 4)
+    >>> samp = sample(N)
+    >>> samp in N.pspace.domain.set
+    True
+    >>> samp_list = sample(N, size=4)
+    >>> [sam in N.pspace.domain.set for sam in samp_list]
+    [True, True, True, True]
+
     """
-    return next(sample_iter(expr, condition, numsamples=1))
+    return next(sample_iter(expr, condition, size=size, numsamples=1))
 
 
-def sample_iter(expr, condition=None, numsamples=S.Infinity, **kwargs):
+def sample_iter(expr, condition=None, size=(), numsamples=S.Infinity, **kwargs):
     """
     Returns an iterator of realizations from the expression given a condition
 
-    expr: Random expression to be realized
-    condition: A conditional expression (optional)
-    numsamples: Length of the iterator (defaults to infinity)
+    Parameters
+    ==========
+
+    expr: Expr
+        Random expression to be realized
+    condition: Expr, optional
+        A conditional expression
+    numsamples: integer, optional
+        Length of the iterator (defaults to infinity)
 
     Examples
     ========
@@ -983,25 +1066,76 @@ def sample_iter(expr, condition=None, numsamples=S.Infinity, **kwargs):
 
     See Also
     ========
-    Sample
+
+    sample
     sampling_P
     sampling_E
     sample_iter_lambdify
     sample_iter_subs
+
     """
     # lambdify is much faster but not as robust
     try:
-        return sample_iter_lambdify(expr, condition, numsamples, **kwargs)
+        return sample_iter_lambdify(expr, condition, size=size, numsamples=numsamples, **kwargs)
     # use subs when lambdify fails
     except TypeError:
-        return sample_iter_subs(expr, condition, numsamples, **kwargs)
+        return sample_iter_subs(expr, condition, size=size, numsamples=numsamples, **kwargs)
 
+def quantile(expr, evaluate=True, **kwargs):
+    r"""
+    Return the :math:`p^{th}` order quantile of a probability distribution.
+
+    Quantile is defined as the value at which the probability of the random
+    variable is less than or equal to the given probability.
+
+    ..math::
+        Q(p) = inf{x \in (-\infty, \infty) such that p <= F(x)}
+
+    Examples
+    ========
+
+    >>> from sympy.stats import quantile, Die, Exponential
+    >>> from sympy import Symbol, pprint
+    >>> p = Symbol("p")
+
+    >>> l = Symbol("lambda", positive=True)
+    >>> X = Exponential("x", l)
+    >>> quantile(X)(p)
+    -log(1 - p)/lambda
+
+    >>> D = Die("d", 6)
+    >>> pprint(quantile(D)(p), use_unicode=False)
+    /nan  for Or(p > 1, p < 0)
+    |
+    | 1       for p <= 1/6
+    |
+    | 2       for p <= 1/3
+    |
+    < 3       for p <= 1/2
+    |
+    | 4       for p <= 2/3
+    |
+    | 5       for p <= 5/6
+    |
+    \ 6        for p <= 1
 
-def sample_iter_lambdify(expr, condition=None, numsamples=S.Infinity, **kwargs):
     """
-    See sample_iter
+    result = pspace(expr).compute_quantile(expr, **kwargs)
 
+    if evaluate and hasattr(result, 'doit'):
+        return result.doit()
+    else:
+        return result
+
+def sample_iter_lambdify(expr, condition=None, size=(), numsamples=S.Infinity, **kwargs):
+    """
     Uses lambdify for computation. This is fast but does not always work.
+
+    See Also
+    ========
+
+    sample_iter
+
     """
     if condition:
         ps = pspace(Tuple(expr, condition))
@@ -1015,19 +1149,16 @@ def sample_iter_lambdify(expr, condition=None, numsamples=S.Infinity, **kwargs):
 
     # Check that lambdify can handle the expression
     # Some operations like Sum can prove difficult
-    try:
-        d = ps.sample()  # a dictionary that maps RVs to values
-        args = [d[rv] for rv in rvs]
-        fn(*args)
-        if condition:
-            given_fn(*args)
-    except Exception:
-        raise TypeError("Expr/condition too complex for lambdify")
+    d = ps.sample(size)  # a dictionary that maps RVs to values
+    args = [d[rv] for rv in rvs]
+    fn(*args)
+    if condition:
+        given_fn(*args)
 
     def return_generator():
         count = 0
         while count < numsamples:
-            d = ps.sample()  # a dictionary that maps RVs to values
+            d = ps.sample(size)  # a dictionary that maps RVs to values
             args = [d[rv] for rv in rvs]
 
             if condition:  # Check that these values satisfy the condition
@@ -1043,11 +1174,15 @@ def sample_iter_lambdify(expr, condition=None, numsamples=S.Infinity, **kwargs):
     return return_generator()
 
 
-def sample_iter_subs(expr, condition=None, numsamples=S.Infinity, **kwargs):
+def sample_iter_subs(expr, condition=None, size=(), numsamples=S.Infinity, **kwargs):
     """
-    See sample_iter
-
     Uses subs for computation. This is slow but almost always works.
+
+    See Also
+    ========
+
+    sample_iter
+
     """
     if condition is not None:
         ps = pspace(Tuple(expr, condition))
@@ -1056,7 +1191,7 @@ def sample_iter_subs(expr, condition=None, numsamples=S.Infinity, **kwargs):
 
     count = 0
     while count < numsamples:
-        d = ps.sample()  # a dictionary that maps RVs to values
+        d = ps.sample(size)  # a dictionary that maps RVs to values
 
         if condition is not None:  # Check that these values satisfy the condition
             gd = condition.xreplace(d)
@@ -1076,9 +1211,11 @@ def sampling_P(condition, given_condition=None, numsamples=1,
 
     See Also
     ========
+
     P
     sampling_E
     sampling_density
+
     """
 
     count_true = 0
@@ -1087,11 +1224,11 @@ def sampling_P(condition, given_condition=None, numsamples=1,
     samples = sample_iter(condition, given_condition,
                           numsamples=numsamples, **kwargs)
 
-    for x in samples:
-        if x != True and x != False:
+    for sample in samples:
+        if sample != True and sample != False:
             raise ValueError("Conditions must not contain free symbols")
 
-        if x:
+        if sample:
             count_true += 1
         else:
             count_false += 1
@@ -1110,6 +1247,7 @@ def sampling_E(expr, given_condition=None, numsamples=1,
 
     See Also
     ========
+
     P
     sampling_P
     sampling_density
@@ -1166,6 +1304,7 @@ def dependent(a, b):
 
     See Also
     ========
+
     independent
     """
     if pspace_independent(a, b):
@@ -1202,6 +1341,7 @@ def independent(a, b):
 
     See Also
     ========
+
     dependent
     """
     return not dependent(a, b)
@@ -1241,23 +1381,119 @@ def rv_subs(expr, symbols=None):
     if not symbols:
         return expr
     swapdict = {rv: rv.symbol for rv in symbols}
-    return expr.xreplace(swapdict)
+    return expr.subs(swapdict)
 
 class NamedArgsMixin(object):
-    _argnames = ()
+    _argnames = ()  # type: tTuple[str, ...]
 
     def __getattr__(self, attr):
         try:
             return self.args[self._argnames.index(attr)]
         except ValueError:
-            raise AttributeError("'%s' object has not attribute '%s'" % (
+            raise AttributeError("'%s' object has no attribute '%s'" % (
                 type(self).__name__, attr))
 
 def _value_check(condition, message):
     """
-    Check a condition on input value.
+    Raise a ValueError with message if condition is False, else
+    return True if all conditions were True, else False.
 
-    Raises ValueError with message if condition is not True
+    Examples
+    ========
+
+    >>> from sympy.stats.rv import _value_check
+    >>> from sympy.abc import a, b, c
+    >>> from sympy import And, Dummy
+
+    >>> _value_check(2 < 3, '')
+    True
+
+    Here, the condition is not False, but it doesn't evaluate to True
+    so False is returned (but no error is raised). So checking if the
+    return value is True or False will tell you if all conditions were
+    evaluated.
+
+    >>> _value_check(a < b, '')
+    False
+
+    In this case the condition is False so an error is raised:
+
+    >>> r = Dummy(real=True)
+    >>> _value_check(r < r - 1, 'condition is not true')
+    Traceback (most recent call last):
+    ...
+    ValueError: condition is not true
+
+    If no condition of many conditions must be False, they can be
+    checked by passing them as an iterable:
+
+    >>> _value_check((a < 0, b < 0, c < 0), '')
+    False
+
+    The iterable can be a generator, too:
+
+    >>> _value_check((i < 0 for i in (a, b, c)), '')
+    False
+
+    The following are equivalent to the above but do not pass
+    an iterable:
+
+    >>> all(_value_check(i < 0, '') for i in (a, b, c))
+    False
+    >>> _value_check(And(a < 0, b < 0, c < 0), '')
+    False
     """
-    if condition == False:
+    from sympy.core.compatibility import iterable
+    from sympy.core.logic import fuzzy_and
+    if not iterable(condition):
+        condition = [condition]
+    truth = fuzzy_and(condition)
+    if truth == False:
         raise ValueError(message)
+    return truth == True
+
+def _symbol_converter(sym):
+    """
+    Casts the parameter to Symbol if it is 'str'
+    otherwise no operation is performed on it.
+
+    Parameters
+    ==========
+
+    sym
+        The parameter to be converted.
+
+    Returns
+    =======
+
+    Symbol
+        the parameter converted to Symbol.
+
+    Raises
+    ======
+
+    TypeError
+        If the parameter is not an instance of both str and
+        Symbol.
+
+    Examples
+    ========
+
+    >>> from sympy import Symbol
+    >>> from sympy.stats.rv import _symbol_converter
+    >>> s = _symbol_converter('s')
+    >>> isinstance(s, Symbol)
+    True
+    >>> _symbol_converter(1)
+    Traceback (most recent call last):
+    ...
+    TypeError: 1 is neither a Symbol nor a string
+    >>> r = Symbol('r')
+    >>> isinstance(r, Symbol)
+    True
+    """
+    if isinstance(sym, str):
+            sym = Symbol(sym)
+    if not isinstance(sym, Symbol):
+        raise TypeError("%s is neither a Symbol nor a string"%(sym))
+    return sym
